#pragma once

#define OBS_DATA_PATH "share/obs"
#define OBS_PLUGIN_PATH "lib/obs-plugins"
#define OBS_PLUGIN_DESTINATION "lib/obs-plugins"

#define GIO_FOUND
#define PULSEAUDIO_FOUND
#define XCB_XINPUT_FOUND
#define ENABLE_WAYLAND

#define OBS_RELEASE_CANDIDATE 0
#define OBS_BETA 0
